package com.qhtx.eta.common.utils;

import com.qhtx.eta.common.eunms.ErrorEnum;

import java.net.*;
import java.util.Collection;
import java.util.Objects;

public class AssertUtils {
    private AssertUtils() {

    }

    /**
     * 断言对象非空。
     *
     * @param object  待检查的对象
     * @param message 断言失败时抛出的异常消息
     * @throws IllegalArgumentException 当对象为空时抛出
     */
    public static void assertNotNull(Object object, String message) {
        if (object == null) {
            throw new IllegalArgumentException(message);
        }
    }

    /**
     * 断言字符串非空且长度不为0。
     *
     * @param str     待检查的字符串
     * @param message 断言失败时抛出的异常消息
     * @throws IllegalArgumentException 当字符串为空或长度为0时抛出
     */
    public static void assertNonEmptyString(String str, String message) {
        if (str == null || str.isEmpty()) {
            throw new IllegalArgumentException(message);
        }
    }

    /**
     * 断言集合非空且至少包含一个元素。
     *
     * @param collection 待检查的集合
     * @param message    断言失败时抛出的异常消息
     * @throws IllegalArgumentException 当集合为空或无元素时抛出
     */
    public static <T extends Collection<?>> void assertNonEmptyCollection(T collection, String message) {
        if (collection == null || collection.isEmpty()) {
            throw new IllegalArgumentException(message);
        }
    }


    /**
     * 断言两个对象相等（使用equals方法比较）。
     *
     * @param expected 期望的对象
     * @param actual   实际的对象
     * @param message  断言失败时抛出的异常消息
     * @throws IllegalArgumentException 当对象不相等时抛出
     */
    public static void assertEquals(Object expected, Object actual, String message) {
        if (!Objects.equals(expected, actual)) {
            throw new IllegalArgumentException(message);
        }
    }

    private static boolean isValidIPv6(String ip) {
        if (ip == null || ip.isEmpty()) {
            return false;
        }

        String regex = "(?:[0-9a-fA-F]{1,4}:){7}[0-9a-fA-F]{1,4}";
        return ip.matches(regex);
    }

    private static boolean isValidIPv4(String ip) {
        if (ip == null || ip.isEmpty()) {
            return false;
        }

        String[] parts = ip.split("\\.");
        if (parts.length != 4) {
            return false;
        }

        for (String part : parts) {
            try {
                int num = Integer.parseInt(part);
                if (num < 0 || num > 255) {
                    return false;
                }
            } catch (NumberFormatException e) {
                return false;
            }
        }
        return true;
    }

    private static void isValidURL(String urlStr, boolean isIPv6) {
        if (urlStr == null || urlStr.isEmpty()) {
        }
        try {
            assertNonEmptyString(urlStr, ErrorEnum.URL_NOT_NULL.getMsg());
            URL url = new URL(urlStr);
            url.toURI();
            String hostname = url.getHost();
            InetAddress inetAddress = InetAddress.getByName(hostname);
            String ipAddress = inetAddress.getHostAddress();
            if (isIPv6) {
                if (!isValidIPv6(ipAddress)) {
                    throw new IllegalArgumentException(ErrorEnum.ILLEGAL_IPV6_ADDRESS.getMsg());
                }
            } else {
                if (!isValidIPv4(ipAddress)) {
                    throw new IllegalArgumentException(ErrorEnum.ILLEGAL_IPV4_ADDRESS.getMsg());
                }
            }
        } catch (MalformedURLException | URISyntaxException | UnknownHostException e) {
            throw new IllegalArgumentException(ErrorEnum.ILLEGAL_URL_ADDRESS.getMsg());
        }
    }

    public static void assertLegalUrl(String url) {
        isValidURL(url, false);
    }

}
