package com.qhtx.eta.common.utils;

import com.alibaba.fastjson.JSONObject;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.common.eunms.ErrorEnum;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;

import java.io.IOException;
import java.time.Instant;
import java.util.Date;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * OKHTTP 客户端
 *
 * @description: 主要对okHttpClient的基础封装，提供发送接口，url和head参数的初始化
 */
@Slf4j
public class HttpClient {
    private static volatile OkHttpClient instance;

    public HttpClient(OkHttpClient okHttpClient) {
        instance = okHttpClient;
    }

    private HttpClient() {
        instance = new OkHttpClient.Builder().build();
    }

    public static OkHttpClient getInstance() {
        if (instance == null) {
            synchronized (HttpClient.class) {
                if (instance == null) {
                    new HttpClient();
                }
            }
        }
        return instance;
    }

    public Response send(Request request) throws IOException {
        if (log.isInfoEnabled()) {
            log.info("api请求接口:{},head信息:{}", request.url(), JSONObject.toJSONString(request.headers()));
        }
        return getInstance().newCall(request).execute();
    }

    public String buildGetUrl(String url, Map<String, Object> params) {
        String path;
        if (params != null && !params.isEmpty()) {
            String paramStr = params.entrySet().stream().map(entry -> entry.getKey() + "=" + entry.getValue()).collect(Collectors.joining("&"));
            path = String.format("%s?%s", url, paramStr);
        } else {
            path = url;
        }
        AssertUtils.assertLegalUrl(path);
        return path;
    }

    public RequestBody createRequestBody(Map<String, Object> params) {
            String jsonStr = JSONObject.toJSONString(params);
            return RequestBody.create(jsonStr,MediaType.get("application/json"));
    }

    public Headers buildHeader(Map<String, ?> params) {
        Headers.Builder headersBuilder = new Headers.Builder();
        for (Map.Entry<String, ?> entry : params.entrySet()) {
            if (entry.getValue() instanceof String) {
                headersBuilder.add(entry.getKey(), (String) entry.getValue());
                continue;
            }
            if (entry.getValue() instanceof Date) {
                headersBuilder.add(entry.getKey(), (Date) entry.getValue());
                continue;
            }
            if (entry.getValue() instanceof Instant) {
                headersBuilder.add(entry.getKey(), (Instant) entry.getValue());
                continue;
            }
            throw new ETAException(ErrorEnum.ILLEGAL_HEAD_PARAM.getCode(), String.format(ErrorEnum.ILLEGAL_HEAD_PARAM.getMsg(), entry.getValue(), entry.getValue().getClass()));
        }
        return headersBuilder.build();
    }
}
