package com.qhtx.eta.domain.api.http;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qhtx.eta.domain.api.http.response.eta.ETAHttpResponse;
import com.qhtx.eta.domain.config.ETAConfigProperties;
import com.qhtx.eta.domain.enums.ETAInterfaceEnum;
import com.qhtx.eta.domain.api.http.response.inner.InnerResponse;
import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.entity.SignatureParam;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.eunms.HttpMethod;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.common.utils.HttpClient;
import com.qhtx.eta.common.utils.SignatureUtil;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import okio.Buffer;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * ETAHttpClient的实现类
 */
@Component
@Slf4j
public class ETAHttpClientImpl implements ETAHttpClient {
    @Resource
    private ETAConfigProperties properties;
    @Resource
    private HttpClient httpClient;

    private static final Map<String, ETAInterfaceEnum> apiInterfaceMap = new ConcurrentHashMap<String, ETAInterfaceEnum>();

    @PostConstruct
    public void init() {
        for (ETAInterfaceEnum etaInterface : ETAInterfaceEnum.values()) {
            apiInterfaceMap.put(etaInterface.getApi(), etaInterface);
        }
    }


    private Request buildETARequest(ETAInterfaceEnum etaInterface, Map<String, Object> params)  {
        String api = etaInterface.getApi();
        String url = properties.getUrl().endsWith("/") ? properties.getUrl().substring(0, properties.getUrl().length() - 1) : properties.getUrl();
        String apiUrl = String.format("%s%s", url, api);
        //生成Header信息
        Headers headers = buildETAHeader();
        if (Objects.requireNonNull(etaInterface.getMethod()) == HttpMethod.GET) {
            apiUrl = httpClient.buildGetUrl(apiUrl, params);
            return new Request.Builder().url(apiUrl).headers(headers).build();
        } else {
            RequestBody body = httpClient.createRequestBody(params);
            Buffer buffer = new Buffer();
            try {
                body.writeTo(buffer);
            }catch(Exception e){
                log.error("请求eta接口异常",e);
            }
            System.out.println(buffer.readUtf8());
            return new Request.Builder().url(apiUrl).headers(headers).post(body).build();
//            throw new ETAException(ErrorEnum.HTTP_METHOD_NOT_ALLOWED);
        }



    }

    private Headers buildETAHeader() {
        String nonce = SignatureUtil.generateNonce();
        long timestamp = SignatureUtil.generateTimestamp();
        String appid = properties.getAppid();
        String secretKey = properties.getSecretKey();
        SignatureParam signatureParam = new SignatureParam.Builder().appid(appid).secretKey(secretKey).nonce(nonce).timestamp(timestamp).build();
        String sign = SignatureUtil.generateSignature(signatureParam);
        if (log.isDebugEnabled()) {
            log.info("签名数据:" + signatureParam.toString());
            log.info("生成签名:" + sign);
        }
        Map<String, String> etaHeader = new HashMap<>();
        etaHeader.put(ETAConstants.NONCE, nonce);
        etaHeader.put(ETAConstants.TIME_STAMP, String.valueOf(timestamp));
        etaHeader.put(ETAConstants.APP_ID, appid);
        etaHeader.put(ETAConstants.SIGNATURE, sign);
        return httpClient.buildHeader(etaHeader);
    }


    public InnerResponse send(ETAInterfaceEnum etaInterface, Map<String, Object> params) throws ETAException {
        Request request = buildETARequest(etaInterface, params);
        try {
            Response response = httpClient.send(request);
            if (response == null || !response.isSuccessful()) {
                if (response != null) {
                    log.warn("{}", String.format(ErrorEnum.API_CALL_ERROR.getMsg(), response.code(), response.message()));
                } else {
                    log.warn("{}", ErrorEnum.RESPONSE_EMPTY_ERROR.getMsg());
                }
                return InnerResponse.errorResponse(response);
            }
            return buildETAResponse(response, etaInterface);
        } catch (IOException e) {
            log.info(e.getMessage());
            return InnerResponse.errorResponse(e.getMessage());
        }
    }

    private InnerResponse buildETAResponse(Response response, ETAInterfaceEnum etaInterfaceEnum) throws IOException {
        if (isSuccessful(response)) {
            ResponseBody responseBody = response.body();
            InnerResponse responseInner = new InnerResponse();
            if (responseBody == null) {
                responseInner.setSuccess(false);
                responseInner.setNeedRetry(false);
                responseInner.setErrorMsg("获取eta应答结果失败,应答结果为空!");
            } else {
                ETAHttpResponse<?, ?> responseData = JSONObject.parseObject(responseBody.string(), etaInterfaceEnum.getResponse());
                if (200 == responseData.getRet()) {
                    List<?> dtos = etaInterfaceEnum.getResponse().cast(responseData).convertDataToDTO();
                    responseInner.setSuccess(true);
                    responseInner.setNeedRetry(false);
                    responseInner.setData(dtos);
                } else {
                    responseInner.setSuccess(false);
                    responseInner.setNeedRetry(false);
                    responseInner.setErrorMsg(String.format("API请求应答错误,api url:[%s],应答结果:[%d:%s]", etaInterfaceEnum.getApi(), responseData.getRet(), StringUtils.isNoneEmpty(responseData.getErrMsg()) ? responseData.getErrMsg() : responseData.getMsg()));
                }
            }
            return responseInner;
        }
        return InnerResponse.emptyResponse();
    }

    private boolean isSuccessful(Response response) {
        return response.isSuccessful();
    }

}
